function [sh, si, sw] = Straight_excitation_variants_resynthesis(x,fs,freq_jitter,plot_specgrams)
% [sh, si, sw] = Straight_excitation_variants_resynthesis(x,fs,freq_jitter,plot_specgrams)
%    x is a speech waveform at sample rate fs.
%    Uses STRAIGHT to decompose into voiced and unvoiced parts, and
%    produce one of three new reconstructions, with a sinusoidal model of
%    the voiced portion:
%    sh is a harmonic reconstruction, intended to be very close
%      to the original.
%    si is an inharmonic version.  Each harmonic is displaced by
%      freq_jitter x f0. (If freq_jitter is a vector, it gives separate offsets
%      for some number of individual harmonics).
%    sw is a simulated whisper with noise replacing the voiced
%      component, and the combined result gently high-pass filtered
%      to be 3 dB down at 1600 Hz, 10 dB down at 1000 Hz, and 40 dB
%      down at 100 Hz.
%    plot_specgrams should be 1 to show spectrograms of the results, 0
%    otherwise
%
%    For Popham, Boebinger, Ellis, Kawahara, McDermott, Nature Communications 2018
%    originally written for Interspeech 2012 (McDermott, Ellis, Kawahara).
% 2018-05-28 Josh McDermott (jhm@mit.edu)

if nargin < 4
    plot_specgrams = 0;
end
if nargin < 3
    freq_jitter = 0;
end

r = exF0candidatesTSTRAIGHTGB(x,fs);
rc = r;
rc = autoF0Tracking(r,x);
rc.vuv = refineVoicingDecision(x,rc);
sourceStructure = aperiodicityRatioSigmoid(x,rc,1,2,0);
filterStructure = exSpectrumTSTRAIGHTGB(x,fs,sourceStructure);
%%

testSubstrate.samplingFrequency = sourceStructure.samplingFrequency;
testSubstrate.sigmoidParameter = sourceStructure.sigmoidParameter;
testSubstrate.vuv = sourceStructure.vuv;
testSubstrate.f0 = sourceStructure.f0;
testSubstrate.temporalPositions = sourceStructure.temporalPositions;
testSubstrate.cutOffListFix = sourceStructure.cutOffListFix;
testSubstrate.targetF0 = sourceStructure.targetF0;
testSubstrate.exponent = sourceStructure.exponent;
testSubstrate.spectrogramSTRAIGHT = filterStructure.spectrogramSTRAIGHT;

testSubstrate.transitionWidth = 0.15;
testSubstrate.sourceOption = (1-0.5*sourceStructure.vuv');

option.deterministicHandleOption.biasFactor = 0;
option.feedingHandleOption.frameRateInSecond = 0.005;
option.deterministicHandleOption.sourceType = 'cosPlusBias';

% Regular resynthesis
syn_Harm = generalSTRAIGHTsynthesisFrameworkR2(@interpFetcherFixRate, ...
    @minimumPhaseResponse,@deterministicExcitationR2,@noiseBurstInFrequencyR2, ...
    @doNothingShifter,testSubstrate,option);

% Simulated whisper
syn_Whisper = generalSTRAIGHTsynthesisFrameworkR2(@interpFetcherFixRate, ...
    @minimumPhaseResponse,@noiseBurstInFrequencyR2,@noiseBurstInFrequencyR2, ...
    @doNothingShifter,testSubstrate,option);

% Inharmonic
option.deterministicHandleOption.biasFactor = freq_jitter;
syn_Inharm = generalSTRAIGHTsynthesisFrameworkR2(@interpFetcherFixRate, ...
    @minimumPhaseResponse,@deterministicExcitationR2,@noiseBurstInFrequencyR2, ...
    @doNothingShifter,testSubstrate,option);

%%


sh = syn_Harm.synthesisOut;
si = syn_Inharm.synthesisOut;

% Simulated whispering is obtained by high-pass filtering the normal
% unvoiced excitation.
%
% For ease of implementation, the filter is applied after the excitation
% and spectrotemporal envelope are recombined
%
% Filter is 2nd order butterworth with 3dB point at 1200 Hz, set
% empirically to resemble whispering
% 2012-08-29 dpwe@ee.columbia.edu
f_c = 1200;
f_nyq = fs/2;
filt_order = 2;
[fb,fa] = butter(filt_order, f_c/f_nyq, 'high');
% but back off the zeros from the unit circle
zero_radius = 0.95;
fb2 = fb(1)*poly([zero_radius;zero_radius]);
sw = filter(fb2,fa,syn_Whisper.synthesisOut);


if plot_specgrams
    
    sg_Harm = stftSpectrogramStructure(syn_Harm.synthesisOut,fs,80,1,'nuttallwin12');
    sg_Whisper = stftSpectrogramStructure(sw,fs,80,1,'nuttallwin12');
    sg_Inharm = stftSpectrogramStructure(syn_Inharm.synthesisOut,fs,80,1,'nuttallwin12');
    
    %figure;
    subplot(311);
    imagesc([0 sg_Harm.temporalPositions(end)],[0 fs/2],max(-90,sg_Harm.dBspectrogram));
    axis([0 sg_Harm.temporalPositions(end) 0 1500]);
    title('Harmonic synthesis');
    axis('xy');colorbar; grid
    
    subplot(312);
    imagesc([0 sg_Whisper.temporalPositions(end)],[0 fs/2],max(-90,sg_Whisper.dBspectrogram));
    axis([0 sg_Whisper.temporalPositions(end) 0 1500]);
    title('Sim Whisper');
    axis('xy');colorbar; grid
    
    subplot(313);
    imagesc([0 sg_Inharm.temporalPositions(end)],[0 fs/2],max(-90,sg_Inharm.dBspectrogram));
    axis([0 sg_Inharm.temporalPositions(end) 0 1500]);
    title(['Inharmonic synthesis']);
    axis('xy');colorbar; grid

end
